﻿class AsteroidManager  
{
	//Constants 
	static var INT_PLAYERSAFEDISTANCE:Number = 100;
	//Arrays
	static var ary_Asteroids:Array;
	//Integers
	static var int_AsteroidCounter:Number;
	static var int_CurrentLevel:Number = 5; // we make this faster so it looks cooler on the front screen
	//Timers
	static var num_NextLevelTimer:Number;
	
	//Constructor
	function AsteroidManager()
	{
		createLevel();
	}
	
	public function downSizeAsteroid(sentAsteroid:MovieClip):Void
	{
		//first we get the current size of the asteroid and store it
		//in a temp variable
		var tempAsteroidSize:Number = sentAsteroid.getAsteroidSize();
		
		//we then need to check if the asteroid can get smaller 
		if(tempAsteroidSize > 1)
		{
			//if so we attach two asteroids one smaller than the current size
			for(var i:Number = 0; i < 2; i++)
			{
				//we send the current asteroids x,y, current direction and size
				attachAsteroid(sentAsteroid.getX(), sentAsteroid.getY(), sentAsteroid.getDirection() + ((Math.random()* 150) - 75), tempAsteroidSize - 1);
			}
		}
		
		//attach the exposion effect and set the x & y to the same as the Asteroid
		//we do this because no matter what the out come is the asteroid has been hit
		_root.particalEffect(sentAsteroid.getX(), sentAsteroid.getY());
		//same reason as above; we need to take this out of the array and remove it
		//from the screen
		removeAsteroid(sentAsteroid);
	}
	
	public function checkHittingAsteroid(sentObject:MovieClip):Void
	{
		//go through all spawned asteroids and see if the sent object has hit any of them
		for(var i:Number = 0; i < int_AsteroidCounter; i++)
		{
			//work out the distance from the current object to the current asteroid
			//and check to see if they have colided
			//we also need to check that there is actually an asteroid in the array position
			if(Math.sqrt(Math.pow(sentObject._x - ary_Asteroids[i]._x, 2) + Math.pow(sentObject._y - ary_Asteroids[i]._y, 2)) <= (sentObject._height/2) + (ary_Asteroids[i]._width/2)
			   && ary_Asteroids[i] != null && ary_Asteroids[i]._alpha >= 100)
			{	
				//update the score
				_root["HUD"].updateScore(6 / ary_Asteroids[i].getAsteroidSize());
				// update the object and the asteroid
				sentObject.takeHit();
				downSizeAsteroid(ary_Asteroids[i]);
				
				//then break from the function and loop because we don't need to check the other asteroids
				return;
			}
		}
	}

	public function fadeAsteroids(sentAction:String):Void
	{
		//Go through all asteroids
		for(var i:Number = 0; i < int_AsteroidCounter; i++)
		{
			//check if which operation needs to be carried out based on the sent string
			if(sentAction == "In")
				ary_Asteroids[i].fadeIn();
			else if(sentAction == "Out")
				ary_Asteroids[i].fadeOut();

		}
	} 
	
	public function updateCurrentLevel(sentLevel:Number):Void
	{
		int_CurrentLevel = sentLevel;
	}
	
	public function clearStartScreen():Void
	{
		fadeAsteroids("Out");
		endLevel();
	}
	
	private static function attachAsteroid(sentX:Number, sentY:Number, sentDirection:Number, sentSize:Number):Void
	{
		//first we attach the movieclip of the asteroid and store in a temp var 
		//for easy access
		var tempAsteroid:MovieClip = _root.attachMovie("Asteroid","Asteroid" + ++int_AsteroidCounter,_root.getNextHighestDepth());
		//we want the asteroids under the HUD and The Outline
		_root.putAsteroidUnder(tempAsteroid);
		//we then push it into the asteroid array
		addToAsteroidArray(tempAsteroid);
		//finially set up the properties for the asteroid
		tempAsteroid.setProperties(sentX, sentY, sentDirection, sentSize, int_CurrentLevel);
	}
	
	private static function removeAsteroid(sentAsteroid:MovieClip):Void
	{
		//First we need to remove the sent asteroid from the array
		removeFromAsteroidArray(sentAsteroid);
		//then we remove it form stage
		sentAsteroid.removeMovieClip();
		//Finially we check to see if it's the end of the level
		checkEndOfLevel();
	}
	
	private static function attachRandomAsteroid():Void
	{//this function was going to be an overload for attachAsteroid but flash doesn't support it -.- 
		
		//In this function we work out a random x, y and direction for the 
		//asteroid and call the other attachAsteroid function sending these
		//properties
		var tempX:Number = getRandomCoordinate();
		var tempY:Number = getRandomCoordinate();
		//to work out the random angle we get a random number times it by 360
		//then take away 180 so it works with flash's rotation of -180 to 180
		var tempDirection:Number = (Math.random() * 360) - 180;
		
		//create the asteroid with the random variables
		attachAsteroid(tempX, tempY, tempDirection, 3);
		
	}
	
	private static function clearAsteroidArray():Void
	{
		//to clear the array we just re-instantiate it
		ary_Asteroids = new Array();
		//clear the current number of asteroids attached
		int_AsteroidCounter = 0;
	}
	
	private static function getRandomCoordinate():Number
	{
		//create a temp var to hold the current random number
		//we times by 400 becuase it's the screen's width and height
		var tempRandomNumber:Number = (Math.random() * 400);
		
		var tempX:Number = 200;
		var tempY:Number = 200;
		
		if(_root["Player"] != undefined)
		{
			tempX = _root["Player"]._x;
			tempY = _root["Player"]._y;
		}
		
		//when getting a random coordinate for the asteroid we need 
		//to make sure that it's not inside the player
		if(tempRandomNumber < tempX - INT_PLAYERSAFEDISTANCE || tempRandomNumber > tempY + INT_PLAYERSAFEDISTANCE)
			return tempRandomNumber;
		else
			//else we start an recusive function unill we get a random number that fits the criteria 
			return getRandomCoordinate();
		
	}
	private static function addToAsteroidArray(sentAsteroid:MovieClip):Void
	{
		//we could just use the push function but the array would become
		//ridiculously over sized and would affect proformace
		for(var i:Number = 0; i < ary_Asteroids.length; i++)
		{
			//check if the current slot is free
			if(ary_Asteroids[i] == null)
			{
			   //if so put the current asteroid in that slot
			   ary_Asteroids[i] = sentAsteroid
			   //return from the function becuase we don't need to run anymore of the function
			   return;
			}
		}
		//if the program has got this far then there is no free space in 
		//the array so we'll just push the asteroid onto the end of the array
		ary_Asteroids.push(sentAsteroid);
		
	}
	
	private static function removeFromAsteroidArray(sentAsteroid:MovieClip):Void
	{
		//we need to compare the sent asteroid to every asteroid in the array
		//to find which one we need to remove
		for(var i:Number = 0; i < ary_Asteroids.length; i++)
		{
			if(ary_Asteroids[i] == sentAsteroid)
			{
				//when we have a match we just set the value at this postion to null
				ary_Asteroids[i] = null;
				//we then break from loop because we don't need to loop anymore
				break;
			}
		}
	}
	
	private static function checkEndOfLevel():Void
	{
		//to know if the level is over we must check if any asteroids are left 
		//in the asteroid array. We can do this by looping through the array and
		//retuning the function if an asteroid is in the current value of i
		for(var i:Number = 0; i < ary_Asteroids.length; i++)
		{
			if(ary_Asteroids[i] != null)
				return
		}

		//if the function gets to here then the level is finished
		endLevel();
	}

	private static function createLevel():Void
	{
		//First we clear the array just to make sure that it doesn't mess with newly attached asteroids
		clearAsteroidArray(); 
		//then attach 4 random asteroids
		for(var i:Number = 0; i < 4; i++)
		{
			attachRandomAsteroid();
		}
		//clear the timer
		clearInterval(num_NextLevelTimer);
		//We may later add a fade-in effect which will go here
	}

	private static function endLevel():Void
	{
		//when the level is over we call the text effect in the HUD to show the level has been finished 
		_root["HUD"].runLevelUpTextEffect();
		//start a timer to start the next level
		num_NextLevelTimer = setInterval(createLevel, 900); 
	}
}
