﻿class HUD extends MovieClip 
{
	//Integers 
	private var int_Score:Number;
	private var int_Level:Number = 1;
	private var int_Lives:Number;
	//Float
	private var flo_CountingScore:Number = 0;
	//Constants 
	private var INT_MAXFUEL:Number = 500;
	private var NUM_TEXTSHRINKRATE:Number = 2;
	//Arrays
	private var ary_LevelScores:Array;
	//Booleans
	private var bool_StartFuelEffect:Boolean = false;
	private var bool_GameOver:Boolean = false;
	private var bool_FadeIn:Boolean = true;
	
	public function onLoad()
	{
		//make sure the players has lives when the game loads
		refillLives();
		//make sure the score is reset
		int_Score = 0;
		int_Level = 1;
		_root.am_AsteroidManager.updateCurrentLevel(int_Level);
		_root["HUD"].mov_Level.txt_Level.text = "Level " + int_Level;
		//make this invisible so it can be faded in
		this._alpha = 0;
	}
	
	public function onEnterFrame()
	{
			
		//Special effect for the top score when it get's updated 
		if(flo_CountingScore <= int_Score)
		{
			//update the counting score until it's the same as the score
			flo_CountingScore += (int_Score - flo_CountingScore) * 0.1;
			//update the onscreen text 
			drawScore();
		}
		
		//we use this to fade in and out the HUD at the start and the end of the game
		if(bool_FadeIn && this._alpha < 100)
			this._alpha += _root.NUM_FADESPEED;
		else if(!bool_FadeIn && this._alpha > 0)
			this._alpha -= _root.NUM_FADESPEED;
		
		//check if the scale is greater than 100 
		if(_root["HUD"].mov_Level._yscale > 100 + NUM_TEXTSHRINKRATE)
		{
			//shrink the text
			_root["HUD"].mov_Level._xscale -= NUM_TEXTSHRINKRATE;
			_root["HUD"].mov_Level._yscale -= NUM_TEXTSHRINKRATE;
		}
	}
	
	public function takeLife()
	{
		//check if the player has any lives
		if(int_Lives > 1)
		{
			//remove the current life images
			_root[this._name]["mov_Life" + int_Lives]._visible = false;
			//Take away a life
			int_Lives--;
		}
		else
		{
			//the last life doesn't remove so we have to clear it now 
			_root[this._name]["mov_Life1"]._visible = false;
			//gameover if the player has no lives
			_root["Player"].gameIsOver();
			attachGameOver();
			bool_FadeIn = false;
		}
	}
	
	public function updateScore(sentScore:Number):Void
	{
		//update the score with the sent value
		//times it by the level to make the feel like
		//they're progressing
		int_Score += sentScore * int_Level;
	}
	public function attachLevelText():Void
	{
		//attach the level text to tell the user the level they are currently on
		_root.attachMovie("LevelText","LevelText",_root.getNextHighestDepth(),{_x:215,_y:68});
		//make sure the text says the right level
		_root["LevelText"].txt_Level.text = "Level " + _root.int_CurrentLevel;
	}
	
	public function attachGameOver():Void
	{
		var tempNumberOfZeros:Number;
		
		//attach the text and button showing the player has finish the game
		_root.attachMovie("GameOver","GameOver",_root.getNextHighestDepth(),{_x:0,_y:0});
		//load in the score for the onscreen text
		_root["GameOver"].int_FinalScore = int_Score;
		//set up a blank variable to add to to get a counting effect
		_root["GameOver"].flo_CounterToFinalScore = 0;
		
		_root["GameOver"].int_FinalLevel = int_Level;
		
		_root["GameOver"]._alpha = 0;
		//now set up the counting and fading in of the movieclip
		//A whole class isn't need for this movieclip so we can set it all up here
		_root["GameOver"].onEnterFrame = function()
		{
			//fade in the game over movieclip
			if(this._alpha < 100)
				this._alpha += _root.NUM_FADESPEED;
				
			//add to the score for effect
			if(Math.round(this.flo_CounterToFinalScore) < this.int_FinalScore)
			{
				this.flo_CounterToFinalScore += (this.int_FinalScore - this.flo_CounterToFinalScore) * 0.1;
			
				//set up the gameover text
				_root["GameOver"].txt_Score.text = Math.round(this.flo_CounterToFinalScore);
				//work out how many zeros we need 
				tempNumberOfZeros =  8 - (_root["GameOver"].txt_Score.text).length;
				//start a loop for adding extra zeros
				for(var i:Number = 0; i < tempNumberOfZeros; i++)
				{
					_root["GameOver"].txt_Score.text = "0" + _root["GameOver"].txt_Score.text;
				}
			
			
				_root["GameOver"].txt_Level.text = this.int_FinalLevel;
				tempNumberOfZeros =  3 - (_root["GameOver"].txt_Level.text).length;
				for(var i:Number = 0; i < tempNumberOfZeros; i++)
				{
					_root["GameOver"].txt_Level.text = "0" + _root["GameOver"].txt_Level.text;
				}
			}
		}
		
		//next we need to set up a function so ge button will take us to the main menu
		_root["GameOver"].btn_ReturnHome.onPress = function()
		{
			//we need to remove these movieclips and attach the main menu
			_root["Player"].removeMovieClip();
			
			//and finially remove the GameOver Text 
			_root["HUD"].removeMovieClip();
			//we actually need to remove the HUD last so all the above function don't get cut off
			_root["GameOver"].onEnterFrame = function()
			{
				if(this._alpha < 0)
				{
					//now load the main menu to the center of the screen
					_root.attachMovie("MainMenu","MainMenu",_root.getNextHighestDepth(),{_x: 0, _y:0});
					_root["MainMenu"]._alpha = 0;
					_root["MainMenu"].onEnterFrame = function()
					{
						if(this._alpha < 100)
							this._alpha += _root.NUM_FADESPEED;
					}
					this.removeMovieClip();
				}
				else
					this._alpha -= _root.NUM_FADESPEED;
			}
		}
	}
	
	public function isGameOver():Boolean
	{
		 return bool_GameOver;
	}
	
	public function fadeOut():Void
	{
		bool_FadeIn = false;
	}
	
	public function nextLevel():Void
	{
		int_Level++;
		_root.am_AsteroidManager.updateCurrentLevel(int_Level);
		_root["HUD"].mov_Level.txt_Level.text = "Level " + int_Level;
	}
	
	public function runLevelUpTextEffect():Void
	{
		//resize and change the value on the text
		_root["HUD"].mov_Level._xscale = 150;
		_root["HUD"].mov_Level._yscale = 150;
		nextLevel();
	}

	public function gameOver():Void
	{
		//when game is over we need to fade out the HUD 
		bool_FadeIn = true; 
		//make sure the ship is disabled
		_root["Player"].gameIsOver();
	}
	
	public function getCurrentLevel():Number 
	{
		return int_Level;
	}
	
	private function drawScore():Void
	{
		//creat a temp variable to store the formatted score
		//initalise it with the current score in string form
		var tempFormattedScore:String = String(Math.round(flo_CountingScore));
		//Add the necessary number of zero to keep the string at a standard length
		for(var i:Number = 0; i < 8 - String(Math.round(flo_CountingScore)).length; i++)
		{
			tempFormattedScore = "0" + tempFormattedScore;
		}
			
		//print out the formatted string
		_root[this._name].txt_Score.text = tempFormattedScore;
	}
	
	private function refillLives():Void
	{
		//Set the lives to 3
		int_Lives = 3;
		//make all the lives visible again
		for(var i:Number = 1; i <= int_Lives; i++)
		{
			_root[this._name]["mov_Life" + i]._visible = true;
		}
	}
	
}