﻿class Player extends MovieClip 
{
	//Constants
	private var INT_ROTATEANGLE:Number = 5;	
	private var FLO_SPEED:Number = 0.1;
	private var INT_MAXVELOCITY:Number = 5;
	private var INT_MAXBULLETS:Number = 5;
	private var INT_BULLETFIREINTERVAL:Number = 10;
	private var FLO_FRICTION:Number = 0.98;
	private var FLO_MINVELOCITY:Number = 0.05;
	//Floats
	private var flo_XVelocity:Number;
	private var flo_YVelocity:Number;
	private var flo_AngleInDegrees:Number;
	//Integers
	private var int_CurrentBullets:Number;
	private var int_BulletIntervalCounter:Number;
	//Arrays
	private var ary_Bullets:Array;
	//Booleans
	private var bool_GameOver:Boolean;
	private var bool_FadeIn:Boolean;
	private var bool_PlayerInvincible:Boolean;
	//Timers
	private var num_InvincibleTimer:Number;
	private var num_FlashTimer:Number;

	public function onLoad()
	{
		resetVelocity();
		int_CurrentBullets = 0;
		int_BulletIntervalCounter = 0;
		ary_Bullets = new Array();
		bool_GameOver = false;
		this._alpha = 100;
		bool_FadeIn = true;
		bool_PlayerInvincible = false;
	}
	
	public function onEnterFrame()
	{
		if(!bool_GameOver)
		{
			//Call function that handles everything to move the player
			movePlayer();
			//call function to handle the player firing
			checkFiringBullet();
			//Call function to detect all collions with walls and other objects
			checkCollions();
		}
		else if(this._alpha < 100 && bool_FadeIn)
			if(bool_FadeIn)
				this._alpha += _root.NUM_FADESPEED;
			else
				bool_GameOver = false;
		else if(this._alpha > 0 && !bool_FadeIn)
			this._alpha -= _root.NUM_FADESPEED;
	}
	
	public function resetVelocity():Void
	{
		flo_XVelocity = 0;
		flo_YVelocity = 0;
	}
	
	public function setPosition(sentX:Number, sentY:Number):Void
	{
		//set the x and y values to the sent parameters
		setX(sentX);
		setY(sentY);
	}
	
	public function	setX(sentX:Number):Void
	{
		this._x = sentX;
	}
	
	public function setY(sentY:Number):Void
	{
		this._y = sentY;
	}
	public function removeBulletFromArray(sentBulletNumber:Number):Void
	{
		ary_Bullets[sentBulletNumber] = null;
	}
	public function takeHit():Void
	{
		//this will handle when the player gets hit#
		if(!bool_PlayerInvincible)
		{
			_root["HUD"].takeLife();
			if(!bool_GameOver)
			{
				makeInvincible();
				setPosition(200, 200);
				resetVelocity();
			}
		}
	}
	
	public function gameIsOver():Void
	{
		bool_GameOver = true;
		bool_FadeIn = false;
	}

	public function makeVincible(sentThis:Player):Void
	{
		this = sentThis;
		//make the player vincible again
		bool_PlayerInvincible = false;
		//clear the timer
		clearInterval(num_InvincibleTimer);
		//clear the flash timer
		clearInterval(num_FlashTimer);
		//make sure the player is visible
		this._alpha = 100;
	}

	public function makePlayerFlash(sentThis:Player):Void 
	{
		this = sentThis;
		if(this._alpha == 100)
			this._alpha = 0;
		else
			this._alpha = 100;
	}
	
	private function movePlayer():Void
	{
		//Controls for the game are both the Arrows and WASD for movement
		// and spacebar for thrust 
		
		//Check if the player is trying to rotate left(Left Arrow OR A)
		if(Key.isDown(Key.LEFT) || Key.isDown(65))
		{
			this._rotation -= INT_ROTATEANGLE;
		}
		//Check if the player is trying to rotate right(Right Arrow OR D)
		else if(Key.isDown(Key.RIGHT) || Key.isDown(68))
		{
			//rotate the player right by the pre-set angle
			this._rotation += INT_ROTATEANGLE;
		}
		//Check if the player is trying to thurst(Spacebar, Up Arrow or W)
		if(Key.isDown(87) || Key.isDown(Key.UP))
		{
			//work out the players current angle in radians
			flo_AngleInDegrees = (this._rotation/360) * 2 * Math.PI;
			//update the player x&y velocity based on it's current angle 
			flo_XVelocity += FLO_SPEED * Math.sin(flo_AngleInDegrees);
			flo_YVelocity -= FLO_SPEED * Math.cos(flo_AngleInDegrees);
		}
		
		//Apply resistance to the ship so it doesn't just keep floating
		flo_XVelocity = flo_XVelocity * FLO_FRICTION;
		flo_YVelocity = flo_YVelocity * FLO_FRICTION;
		
		//we need to limit the max velocity of the player so the player can't fly out of control
		if(flo_XVelocity >= INT_MAXVELOCITY)
			flo_XVelocity = INT_MAXVELOCITY;
		else if(flo_XVelocity <= -INT_MAXVELOCITY)
			flo_XVelocity = -INT_MAXVELOCITY;
		
		if(flo_YVelocity >= INT_MAXVELOCITY)
			flo_YVelocity = INT_MAXVELOCITY;
		else if(flo_YVelocity <= -INT_MAXVELOCITY)
			flo_YVelocity = -INT_MAXVELOCITY;
		
		//if the ship velocity get really low stop it
		if(flo_XVelocity < FLO_MINVELOCITY && flo_XVelocity > -FLO_MINVELOCITY)
			flo_XVelocity = 0;
		if(flo_YVelocity < FLO_MINVELOCITY && flo_YVelocity > -FLO_MINVELOCITY)
			flo_YVelocity = 0;
		
		//update the players x and y positions applying gravity
		this._x += flo_XVelocity;
		this._y += flo_YVelocity;
	}
	
	private function checkFiringBullet():Void
	{
		//call a function to see if there is any space for a new bullet
		var tempCurrentBullets:Number = checkSpaceInBulletArray();
		//check if the player is pressing fire and that the the player
		//has not used all their bullets and the fire interval has passed
		if(Key.isDown(Key.SPACE) &&  tempCurrentBullets < INT_MAXBULLETS && int_BulletIntervalCounter >= INT_BULLETFIREINTERVAL)
		{
			//rest the bullet fire interval
			int_BulletIntervalCounter = 0;
			//attach a new bullet and store in the array
			ary_Bullets[tempCurrentBullets] = _root.attachMovie("Bullet","Bullet" + tempCurrentBullets,_root.getNextHighestDepth());
			//set the properties for the new bullet
			ary_Bullets[tempCurrentBullets].setProperties(this._x, this._y, this._rotation, tempCurrentBullets);
		}
		else
			//increment the counter 
			int_BulletIntervalCounter++
	}
	
	private function checkCollions():Void
	{
		//When the player goes of one side of the screen the should reappear on the otherside
		
		//first we check if the player has gone of the left side of the screen
		//we also acount for the point of reference being the middle of the movieclip
		//and adjusting the value so the whole ship is off screen before repositioning it
		if(this._x < 0 - (this._width/2) + 4)
			//again adjust the ship position when attaching it off screen
			setX(Stage.width + (this._width/2)- 4);
		//We do the same again but for the right and side
		else if(this._x > Stage.width + (this._width/2) - 4)
			//again adjust the ship position when attaching it off screen
			setX(0 - (this._width/2)+ 4);
		
		//We then repeat the same process as above but only checking if the player has gone 
		//off the top or the bottom
		if(this._y < 0 - (this._width/2) + 4)
			setY(Stage.height + (this._height/2)- 4);
		else if(this._y > Stage.height + (this._width/2) - 4)
			setY(0 - (this._height/2)+ 4);
			
		//check if the player is hitting an asteroid
		_root.am_AsteroidManager.checkHittingAsteroid(this);
		
	}
	
	private function checkSpaceInBulletArray():Number
	{
		var i:Number;
		//start a loop through the bullet array looking for free space
		for(i = 0; i < INT_MAXBULLETS;i++)
		{
			//check if a bullet is in the current spot
			if(ary_Bullets[i] == undefined || ary_Bullets[i] == null)
			{
				//if not we can return the current value of i
				return i;
			}
		}
		//if there is no space the function will get to this point so return one more than the bullet limit
		return INT_MAXBULLETS + 1;
	}

	private function makeInvincible():Void
	{
		//Change the flag so the player can't be damaged
		bool_PlayerInvincible = true; 
		//Set a timer to so the player can be damaged again
		num_InvincibleTimer = setInterval(makeVincible, 1500, this);
		//start a flashing timer, this timer will be cleared by the timer above
		num_FlashTimer = setInterval(makePlayerFlash, 100, this);
	}
}